package cn.iocoder.yudao.module.dms.service.checktemplateclassify;

import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.dms.controller.admin.checktemplateclassify.vo.DmsCheckTemplateClassifyListReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.checktemplateclassify.vo.DmsCheckTemplateClassifySaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktemplateclassify.DmsCheckTemplateClassifyDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.checktemplateclassify.DmsCheckTemplateClassifyMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.*;

/**
 * 点巡检模板分类 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsCheckTemplateClassifyServiceImpl implements DmsCheckTemplateClassifyService {

    @Resource
    private DmsCheckTemplateClassifyMapper checkTemplateClassifyMapper;

    @Override
    public Long createCheckTemplateClassify(DmsCheckTemplateClassifySaveReqVO createReqVO) {
        // 校验父设备设施分类ID的有效性
        validateParentCheckTemplateClassify(null, createReqVO.getParentId());
        // 校验设备设施分类名称的唯一性
        validateCheckTemplateClassifyNameUnique(null, createReqVO.getParentId(), createReqVO.getName());

        // 插入
        DmsCheckTemplateClassifyDO checkTemplateClassify = BeanUtils.toBean(createReqVO, DmsCheckTemplateClassifyDO.class);
        checkTemplateClassify.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        checkTemplateClassifyMapper.insert(checkTemplateClassify);
        // 返回
        return checkTemplateClassify.getId();
    }

    @Override
    public void updateCheckTemplateClassify(DmsCheckTemplateClassifySaveReqVO updateReqVO) {
        // 校验存在
        validateCheckTemplateClassifyExists(updateReqVO.getId());
        // 校验父设备设施分类ID的有效性
        validateParentCheckTemplateClassify(updateReqVO.getId(), updateReqVO.getParentId());
        // 校验设备设施分类名称的唯一性
        validateCheckTemplateClassifyNameUnique(updateReqVO.getId(), updateReqVO.getParentId(), updateReqVO.getName());

        // 更新
        DmsCheckTemplateClassifyDO updateObj = BeanUtils.toBean(updateReqVO, DmsCheckTemplateClassifyDO.class);
        checkTemplateClassifyMapper.updateById(updateObj);
    }

    @Override
    public void deleteCheckTemplateClassify(Long id) {
        // 校验存在
        validateCheckTemplateClassifyExists(id);
        // 校验是否有子点巡检模板分类
        if (checkTemplateClassifyMapper.selectCountByParentId(id) > 0) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_EXITS_CHILDREN);
        }
        // 删除
        checkTemplateClassifyMapper.deleteById(id);
    }

    private void validateCheckTemplateClassifyExists(Long id) {
        if (checkTemplateClassifyMapper.selectById(id) == null) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_NOT_EXISTS);
        }
    }

    private void validateParentCheckTemplateClassify(Long id, Long parentId) {
        if (parentId == null || DmsCheckTemplateClassifyDO.PARENT_ID_ROOT.equals(parentId)) {
            return;
        }
        // 1. 不能设置自己为父点巡检模板分类
        if (Objects.equals(id, parentId)) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_PARENT_ERROR);
        }
        // 2. 父点巡检模板分类不存在
        DmsCheckTemplateClassifyDO parentCheckTemplateClassify = checkTemplateClassifyMapper.selectById(parentId);
        if (parentCheckTemplateClassify == null) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_PARENT_NOT_EXITS);
        }
        // 3. 递归校验父点巡检模板分类，如果父点巡检模板分类是自己的子点巡检模板分类，则报错，避免形成环路
        if (id == null) { // id 为空，说明新增，不需要考虑环路
            return;
        }
        for (int i = 0; i < Short.MAX_VALUE; i++) {
            // 3.1 校验环路
            parentId = parentCheckTemplateClassify.getParentId();
            if (Objects.equals(id, parentId)) {
                throw exception(CHECK_TEMPLATE_CLASSIFY_PARENT_IS_CHILD);
            }
            // 3.2 继续递归下一级父点巡检模板分类
            if (parentId == null || DmsCheckTemplateClassifyDO.PARENT_ID_ROOT.equals(parentId)) {
                break;
            }
            parentCheckTemplateClassify = checkTemplateClassifyMapper.selectById(parentId);
            if (parentCheckTemplateClassify == null) {
                break;
            }
        }
    }

    private void validateCheckTemplateClassifyNameUnique(Long id, Long parentId, String name) {
        DmsCheckTemplateClassifyDO checkTemplateClassify = checkTemplateClassifyMapper.selectByParentIdAndName(parentId, name);
        if (checkTemplateClassify == null) {
            return;
        }
        // 如果 id 为空，说明不用比较是否为相同 id 的点巡检模板分类
        if (id == null) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_NAME_DUPLICATE);
        }
        if (!Objects.equals(checkTemplateClassify.getId(), id)) {
            throw exception(CHECK_TEMPLATE_CLASSIFY_NAME_DUPLICATE);
        }
    }

    @Override
    public DmsCheckTemplateClassifyDO getCheckTemplateClassify(Long id) {
        return checkTemplateClassifyMapper.selectById(id);
    }

    @Override
    public List<DmsCheckTemplateClassifyDO> getCheckTemplateClassifyList(DmsCheckTemplateClassifyListReqVO listReqVO) {
        return checkTemplateClassifyMapper.selectList(listReqVO);
    }

    /**
     * 获得点巡检模板分类精简列表
     *
     * @return 点巡检模板分类列表
     */
    @Override
    public List<DmsCheckTemplateClassifyDO> getSimpleCheckTemplateClassifyList() {
        return checkTemplateClassifyMapper.selectSimpleList();
    }

}
