package cn.iocoder.yudao.module.dms.service.deviceclassify;

import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsDeviceClassifyListReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsDeviceClassifySaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.deviceclassify.DmsDeviceClassifyDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.deviceclassify.DmsDeviceClassifyMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.*;

/**
 * 设备分类 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsDeviceClassifyServiceImpl implements DmsDeviceClassifyService {

    @Resource
    private DmsDeviceClassifyMapper dmsDeviceClassifyMapper;

    @Override
    public Long createDeviceClassify(DmsDeviceClassifySaveReqVO createReqVO) {
        // 校验父分类ID的有效性
        validateParentDeviceClassify(null, createReqVO.getParentId());
        // 校验设备设施分类名称的唯一性
        validateDeviceClassifyNameUnique(null, createReqVO.getParentId(), createReqVO.getName());

        // 插入
        DmsDeviceClassifyDO deviceClassify = BeanUtils.toBean(createReqVO, DmsDeviceClassifyDO.class);
        deviceClassify.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        dmsDeviceClassifyMapper.insert(deviceClassify);
        // 返回
        return deviceClassify.getId();
    }

    @Override
    public void updateDeviceClassify(DmsDeviceClassifySaveReqVO updateReqVO) {
        // 校验存在
        validateDeviceClassifyExists(updateReqVO.getId());
        // 校验父分类ID的有效性
        validateParentDeviceClassify(updateReqVO.getId(), updateReqVO.getParentId());
        // 校验设备设施分类名称的唯一性
        validateDeviceClassifyNameUnique(updateReqVO.getId(), updateReqVO.getParentId(), updateReqVO.getName());

        // 更新
        DmsDeviceClassifyDO updateObj = BeanUtils.toBean(updateReqVO, DmsDeviceClassifyDO.class);
        dmsDeviceClassifyMapper.updateById(updateObj);
    }

    @Override
    public void deleteDeviceClassify(Long id) {
        // 校验存在
        validateDeviceClassifyExists(id);
        // 校验是否有子设备分类
        if (dmsDeviceClassifyMapper.selectCountByParentId(id) > 0) {
            throw exception(DEVICE_CLASSIFY_EXITS_CHILDREN);
        }
        // 删除
        dmsDeviceClassifyMapper.deleteById(id);
    }

    private void validateDeviceClassifyExists(Long id) {
        if (dmsDeviceClassifyMapper.selectById(id) == null) {
            throw exception(DEVICE_CLASSIFY_NOT_EXISTS);
        }
    }

    private void validateParentDeviceClassify(Long id, Long parentId) {
        if (parentId == null || DmsDeviceClassifyDO.PARENT_ID_ROOT.equals(parentId)) {
            return;
        }
        // 1. 不能设置自己为父设备分类
        if (Objects.equals(id, parentId)) {
            throw exception(DEVICE_CLASSIFY_PARENT_ERROR);
        }
        // 2. 父设备分类不存在
        DmsDeviceClassifyDO parentDeviceClassify = dmsDeviceClassifyMapper.selectById(parentId);
        if (parentDeviceClassify == null) {
            throw exception(DEVICE_CLASSIFY_PARENT_NOT_EXITS);
        }
        // 3. 递归校验父设备分类，如果父设备分类是自己的子设备分类，则报错，避免形成环路
        if (id == null) { // id 为空，说明新增，不需要考虑环路
            return;
        }
        for (int i = 0; i < Short.MAX_VALUE; i++) {
            // 3.1 校验环路
            parentId = parentDeviceClassify.getParentId();
            if (Objects.equals(id, parentId)) {
                throw exception(DEVICE_CLASSIFY_PARENT_IS_CHILD);
            }
            // 3.2 继续递归下一级父设备分类
            if (parentId == null || DmsDeviceClassifyDO.PARENT_ID_ROOT.equals(parentId)) {
                break;
            }
            parentDeviceClassify = dmsDeviceClassifyMapper.selectById(parentId);
            if (parentDeviceClassify == null) {
                break;
            }
        }
    }

    private void validateDeviceClassifyNameUnique(Long id, Long parentId, String name) {
        DmsDeviceClassifyDO deviceClassify = dmsDeviceClassifyMapper.selectByParentIdAndName(parentId, name);
        if (deviceClassify == null) {
            return;
        }
        // 如果 id 为空，说明不用比较是否为相同 id 的设备分类
        if (id == null) {
            throw exception(DEVICE_CLASSIFY_NAME_DUPLICATE);
        }
        if (!Objects.equals(deviceClassify.getId(), id)) {
            throw exception(DEVICE_CLASSIFY_NAME_DUPLICATE);
        }
    }

    @Override
    public DmsDeviceClassifyDO getDeviceClassify(Long id) {
        return dmsDeviceClassifyMapper.selectById(id);
    }

    @Override
    public List<DmsDeviceClassifyDO> getDeviceClassifyList(DmsDeviceClassifyListReqVO listReqVO) {
        return dmsDeviceClassifyMapper.selectList(listReqVO);
    }

    /**
     * 获得设备分类精简列表
     *
     * @return 设备分类精简列表
     */
    @Override
    public List<DmsDeviceClassifyDO> getSimpleDeviceClassifyList() {
        return dmsDeviceClassifyMapper.selectSimpleList();
    }

}
